@@ -1,5 +1,5 @@
 /*
- * $Id: tools.c 5432 2011-08-22 12:55:04Z matthijs $
+ * $Id: tools.c 5849 2011-11-14 08:43:44Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -318,7 +318,12 @@ tools_audit(zone_type* zone, char* working_dir, char* cfg_filename)
         free((void*)inbound);
 
         if (error) {
+            ods_log_error("[%s] audit failed for zone %s", tools_str,
+                zone->name);
             status = ODS_STATUS_ERR;
+        } else {
+            ods_log_info("[%s] audit passed for zone %s", tools_str,
+                zone->name);
         }
         end = time(NULL);
         if (status == ODS_STATUS_OK && zone->stats) {
